# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from rest_framework import response, serializers, views, viewsets
from rest_framework.decorators import api_view, detail_route, list_route


class SimpleSerializer(serializers.Serializer):
    pk = serializers.ReadOnlyField()


class BasicView(views.APIView):
    queryset = get_user_model().objects.none()
    def get(self, request, *args, **kwargs):
        return response.Response({'method': 'GET'})

    def post(self, request, *args, **kwargs):
        return response.Response({'method': 'POST', 'data': request.data})


@api_view(http_method_names=["post", "get"])
def function_endpoint(request):
    return response.Response({'method': request.method})


class SimpleViewSet(viewsets.GenericViewSet):
    queryset = get_user_model().objects.none()
    serializer_class = SimpleSerializer

    @list_route(methods=["patch", "put"])
    def patch_put(self, request):
        return response.Response({'method': request.method})


class SimpleModelViewSet(viewsets.ModelViewSet):
    queryset = get_user_model().objects.none()
    serializer_class = SimpleSerializer

    @list_route(methods=["delete", "get"])
    def some_method(self, request):
        return response.Response({'method': request.method})

    @detail_route(methods=["get", "post", "patch", "put"])
    def some_detail_metod(self, request):
        return response.Response({'method': request.method})



# -*- coding: utf-8 -*-


class Scope(object):
    """
    A base scope.
    This indicates a permission access.
    The `identifier` should be unique.
    """
    identifier = None

    def __init__(self, identifier):
        self.identifier = identifier

    def get_description(self):
        return None

    def __str__(self):
        return "Base scope: {}".format(self.identifier)



# -*- coding: utf-8 -*-
from django.conf.urls import include, url
from django.test.utils import override_settings
from rest_framework import routers

from rest_jwt_permission.providers import get_permission_providers
from rest_jwt_permission.scopes import APIScope, Scope, SimpleScope, get_all_permission_providers_scopes
from rest_jwt_permission_tests.factories import BasicView, SimpleModelViewSet, SimpleViewSet, function_endpoint


def setup_function(fn):
    get_permission_providers.cache_clear()


router = routers.DefaultRouter()
router.register(r'simple-viewset', SimpleViewSet)
router.register(r'model-viewset', SimpleModelViewSet)

urlpatterns = [
    url(r'^simple/$', BasicView.as_view()),
    url(r'^function/$', function_endpoint),
    url(r'^', include(router.urls)),
]


def test_api_endpoint_scope_permission_provider():
    with override_settings(
        ROOT_URLCONF="rest_jwt_permission_tests.providers.test_api_endpoint_provider",
        REST_JWT_PERMISSION={
            "SCOPE_PROVIDERS": [
                "rest_jwt_permission.providers.APIEndpointScopeProvider"
            ]
        }
    ):
        scopes = get_all_permission_providers_scopes()
        expected_identifiers = [
            'apirootview:get',
            'function_endpoint:get',
            'function_endpoint:post',
            'simplemodelviewset:list:get',
            'simplemodelviewset:create:post',
            'simplemodelviewset:some_method:get',
            'simplemodelviewset:some_method:delete',
            'simplemodelviewset:retrieve:get',
            'simplemodelviewset:update:put',
            'simplemodelviewset:partial_update:patch',
            'simplemodelviewset:destroy:delete',
            'simplemodelviewset:some_detail_metod:get',
            'simplemodelviewset:some_detail_metod:post',
            'simplemodelviewset:some_detail_metod:put',
            'simplemodelviewset:some_detail_metod:patch',
            'simpleviewset:patch_put:put',
            'simpleviewset:patch_put:patch',
            'basicview:get',
            'basicview:post'
        ]
        scope_ids = [s.identifier for s in scopes]
        assert len(scope_ids) == len(expected_identifiers)
        assert set(scope_ids) == set(expected_identifiers)
        assert all([isinstance(s, Scope) for s in scopes])
        assert all([isinstance(s, APIScope) for s in scopes])
        assert not any([isinstance(s, SimpleScope) for s in scopes])



# -*- coding: utf-8 -*-
from django.utils import module_loading
from django.utils.lru_cache import lru_cache

from rest_jwt_permission.settings import get_setting


@lru_cache()
def get_permission_providers():
    providers = []

    for provider in get_setting("SCOPE_PROVIDERS", []):
        provider_class = module_loading.import_string(provider)
        providers.append(provider_class())

    return providers



# -*- coding: utf-8 -*-
from .base import Scope


class APIScope(Scope):
    path = None
    method = None
    view_class = None

    def __init__(self, identifier, path, method, view_class):
        super(APIScope, self).__init__(identifier)
        self.path = path
        self.method = method
        self.view_class = view_class

    def get_description(self):
        return self.path

    def __str__(self):
        return "API endpoint scope: {} - [{}]{} - {}".format(
            self.identifier, self.method.upper(), self.path, self.view_class.__name__
        )



# -*- coding: utf-8 -*-
import jwt
from django.utils.text import force_text

from rest_jwt_permission.handlers import get_jwt_payload_from_request, get_payload_from_scopes, get_scopes_from_payload
from rest_jwt_permission.scopes import SimpleScope


def test_get_payload_from_scopes_from_payload():
    scopes = [
        SimpleScope("scope-1", "desc1"),
        SimpleScope("scope-2", "desc2"),
        SimpleScope("scope-3", "desc3")
    ]

    payload = get_payload_from_scopes(scopes)
    assert payload == {"scopes": ["scope-1", "scope-2", "scope-3"]}

    payload.update({
        "add": "more",
        "stuff": "here",
        "garbage": [1, 2, 3, 4, 5]
    })

    assert get_scopes_from_payload(payload) == ["scope-1", "scope-2", "scope-3"]


def test_get_jwt_payload_from_request(rf):
    key = "super-secret"  # defined at conftest.py
    scopes = [
        SimpleScope("scope-1", "desc1"),
        SimpleScope("scope-2", "desc2"),
        SimpleScope("scope-3", "desc3")
    ]
    payload = get_payload_from_scopes(scopes)
    encoded = jwt.encode(payload, key, algorithm='HS256')

    request = rf.post("/", HTTP_AUTHORIZATION="JWT {}".format(force_text(encoded)))
    request_payload = get_jwt_payload_from_request(request)
    assert payload == request_payload



# -*- coding: utf-8 -*-
from django.utils import module_loading

REST_JWT_PERMISSION = {
    # List of scope  providers
    "SCOPE_PROVIDERS": [
        "rest_jwt_permission.providers.APIEndpointScopeProvider",
        "rest_jwt_permission.providers.AdminScopeProvider"
    ],

    # Handler function to get the payload with scopes to be injected in JWT payload
    "GET_PAYLOAD_FROM_SCOPES_HANDLER": (
        "rest_jwt_permission.handlers.get_payload_from_scopes"
    ),

    # Handler function to get the scopes from a JWT payload
    "GET_SCOPES_FROM_PAYLOAD_HANDLER": (
        "rest_jwt_permission.handlers.get_scopes_from_payload"
    ),

    # Handler function to get JWT payload from Request
    "GET_PAYLOAD_FROM_REQUEST_HANDLER": (
        "rest_jwt_permission.handlers.get_jwt_payload_from_request"
    ),

    # Payload key that will contain the scopes
    "JWT_PAYLOAD_SCOPES_KEY": "scopes"
}


def get_setting(setting_key, default=None):
    from django.conf import settings as django_settings
    settings = getattr(django_settings, "REST_JWT_PERMISSION", {})
    return settings.get(setting_key, REST_JWT_PERMISSION.get(setting_key, default))


def get_imported_setting(setting_key):
    return module_loading.import_string(get_setting(setting_key))



# -*- coding: utf-8 -*-
import inspect

from django.utils.text import slugify


def get_role_for(method, action=None):
    if action:
        return "{}:{}".format(action, method.lower())
    return method.lower()


def get_view_id(view):
    view_class = None
    view_id = None

    if inspect.isclass(view):
        view_class = view
    else:
        view_class = view.__class__

    # check whether the view has the special get_view_permission_id classmethod
    if hasattr(view_class, "get_view_permission_id") and callable(view_class.get_view_permission_id):
        # check whether that is a class method
        if (inspect.ismethod(view_class.get_view_permission_id) and
                view_class.get_view_permission_id.__self__ is view_class):
            view_id = slugify(view_class.get_view_permission_id())
        else:
            view_id = slugify(view_class().get_view_permission_id())

    if not view_id:
        view_id = slugify(view_class.__name__)

    return view_id


def get_view_role(view, role):
    return "{}:{}".format(get_view_id(view), role)



# -*- coding: utf-8 -*-
from .base import Scope


class SimpleScope(Scope):
    description = None

    def __init__(self, identifier, description):
        super(SimpleScope, self).__init__(identifier)
        self.description = description

    def get_description(self):
        return self.description

    def __str__(self):
        return "Simple scope: {}".format(self.identifier)



# -*- coding: utf-8 -*-
import pytest
from django.test.utils import override_settings

from rest_jwt_permission.providers import (
    AdminScopeProvider, APIEndpointScopeProvider, get_permission_providers
)
from rest_jwt_permission.providers.base import ScopeProviderBase


def setup_function(fn):
    get_permission_providers.cache_clear()


def test_base_provider():
    with pytest.raises(NotImplementedError):
        ScopeProviderBase().get_available_scopes()


def test_get_permission_providers():
    # default providers
    providers = get_permission_providers()
    assert len(providers) == 2
    assert AdminScopeProvider in [provider.__class__ for provider in providers]
    assert APIEndpointScopeProvider in [provider.__class__ for provider in providers]

    # clear the lru_cache
    get_permission_providers.cache_clear()

    with override_settings(REST_JWT_PERMISSION={
        "SCOPE_PROVIDERS": [
            "rest_jwt_permission.providers.APIEndpointScopeProvider"
        ]
    }):
        providers = get_permission_providers()
        assert len(providers) == 1
        assert providers[0].__class__ == APIEndpointScopeProvider

    # clear the lru_cache
    get_permission_providers.cache_clear()

    with override_settings(REST_JWT_PERMISSION={
        "SCOPE_PROVIDERS": [
            "rest_jwt_permission.providers.AdminScopeProvider"
        ]
    }):
        providers = get_permission_providers()
        assert len(providers) == 1
        assert providers[0].__class__ == AdminScopeProvider



# -*- coding: utf-8 -*-
from django.utils.lru_cache import lru_cache
from rest_framework.schemas import EndpointInspector

from rest_jwt_permission.utils import get_role_for, get_view_role

from .base import ScopeProviderBase


class APIEndpointScopeProvider(ScopeProviderBase):
    @lru_cache()
    def get_available_scopes(self):
        from rest_jwt_permission.scopes import APIScope

        epi = EndpointInspector()
        scopes = []

        for (path, method, callback) in epi.get_api_endpoints():
            view_class = callback.cls
            method = method.lower()
            action = callback.actions[method] if (hasattr(callback, "actions") and callback.actions) else None
            role = get_role_for(method, action)
            scope = get_view_role(view_class, role)
            scopes.append(APIScope(scope, path, method, view_class))

        return scopes



# -*- coding: utf-8 -*-
from .admin import AdminScopeProvider
from .api_endpoint import APIEndpointScopeProvider
from .utils import get_permission_providers

__all__ = [
    "APIEndpointScopeProvider",
    "AdminScopeProvider",
    "get_permission_providers"
]



# -*- coding: utf-8 -*-
from rest_framework.permissions import BasePermission

from .settings import get_imported_setting
from .utils import get_role_for, get_view_role


class JWTAPIPermission(BasePermission):
    """
    Returns whether the scope is inside the JWT payload
    """
    def has_permission(self, request, view):
        get_jwt_payload_from_request = get_imported_setting("GET_PAYLOAD_FROM_REQUEST_HANDLER")
        get_scopes_from_payload = get_imported_setting("GET_SCOPES_FROM_PAYLOAD_HANDLER")

        payload = get_jwt_payload_from_request(request)

        if not payload:
            return False

        payload_scopes = get_scopes_from_payload(payload)
        role = get_role_for(request.method.lower(), getattr(view, "action", None))
        scope = get_view_role(view, role)

        return scope in payload_scopes



# -*- coding: utf-8 -*-
from .base import Scope
from .api import APIScope
from .simple import SimpleScope
from .utils import get_all_permission_providers_scopes


__all__ = [
    "APIScope",
    "Scope",
    "SimpleScope",
    "get_all_permission_providers_scopes"
]



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand

from rest_jwt_permission.scopes import get_all_permission_providers_scopes


class Command(BaseCommand):
    def handle(self, *args, **options):
        for x in get_all_permission_providers_scopes():
            print(x)



# -*- coding: utf-8 -*-
from django.test.utils import override_settings

from rest_jwt_permission.providers import get_permission_providers
from rest_jwt_permission.scopes import APIScope, Scope, SimpleScope, get_all_permission_providers_scopes


def setup_function():
    get_permission_providers.cache_clear()


def test_admin_scope_permission_provider():
    with override_settings(
        REST_JWT_PERMISSION={
            "SCOPE_PROVIDERS": [
                "rest_jwt_permission.providers.AdminScopeProvider"
            ]
        }
    ):
        scopes = get_all_permission_providers_scopes()
        expected_identifiers = [
            'superuser'
        ]
        scope_ids = [s.identifier for s in scopes]
        assert len(scope_ids) == len(expected_identifiers)
        assert set(scope_ids) == set(expected_identifiers)
        assert all([isinstance(s, Scope) for s in scopes])
        assert all([isinstance(s, SimpleScope) for s in scopes])
        assert not any([isinstance(s, APIScope) for s in scopes])



# -*- coding: utf-8 -*-
from rest_jwt_permission.scopes import APIScope, Scope, SimpleScope
from rest_jwt_permission_tests.factories import SimpleModelViewSet


def test_base_scope():
    assert Scope("ident1").identifier == "ident1"
    assert Scope("ident1").get_description() is None
    assert str(Scope("ident1")) == "Base scope: ident1"


def test_simple_scope():
    scope = SimpleScope("ident2", "my description")
    assert scope.identifier == "ident2"
    assert scope.get_description() == "my description"
    assert str(scope) == "Simple scope: ident2"


def test_api_scope():
    scope = APIScope("simple-api", "/path/action/", "post", SimpleModelViewSet)
    assert scope.identifier == "simple-api"
    assert scope.get_description() == "/path/action/"
    assert str(scope) == "API endpoint scope: simple-api - [POST]/path/action/ - SimpleModelViewSet"



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

try:
    long_description = open("README.md").read()
except IOError:
    long_description = ""

setup(
    name="rest-jwt-permission",
    version="0.2.0",
    description="Django Rest Framework JWT Permissions",
    license="MIT",
    author="Christian Hess",
    author_email="christianhess.rlz@gmail.com",
    packages=find_packages(),
    install_requires=[],
    long_description=long_description,
    url="https://github.com/chessbr/rest-jwt-permission",
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 2.7",
        "Environment :: Web Environment",
        "Topic :: Internet :: WWW/HTTP",
        "Intended Audience :: Developers"
    ]
)



# -*- coding: utf-8 -*-
import itertools

from rest_jwt_permission.providers import get_permission_providers


def get_all_permission_providers_scopes():
    providers_scopes = [provider.get_available_scopes() for provider in get_permission_providers()]
    return [scope for scope in itertools.chain(*providers_scopes)]



# -*- coding: utf-8 -*-
import jwt
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from rest_framework import exceptions
from rest_framework.authentication import get_authorization_header

from .settings import get_setting


def get_payload_from_scopes(scopes):
    """
    Get a dict to be used in JWT payload.
    Just merge this dict with the JWT payload.

    :type roles list[rest_jwt_permission.scopes.Scope]
    :return dictionary to be merged with the JWT payload
    :rtype dict
    """
    return {
        get_setting("JWT_PAYLOAD_SCOPES_KEY"): [scope.identifier for scope in scopes]
    }


def get_scopes_from_payload(payload):
    """
    Get a list of Scope from the JWT payload

    :type payload dict
    :rtype list[rest_jwt_permission.scopes.Scope]
    """
    return payload.get(get_setting("JWT_PAYLOAD_SCOPES_KEY"), [])


def get_jwt_payload_from_request(request):
    # lazy load this if one want to use other rest lib for JWT
    from rest_framework_jwt.settings import api_settings
    jwt_decode_handler = api_settings.JWT_DECODE_HANDLER

    auth = get_authorization_header(request).split()
    auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

    if not auth:
        if api_settings.JWT_AUTH_COOKIE:
            return request.COOKIES.get(api_settings.JWT_AUTH_COOKIE)
        return None

    if smart_text(auth[0].lower()) != auth_header_prefix:
        return None

    if len(auth) == 1:
        raise exceptions.PermissionDenied(_('No credentials provided.'))
    elif len(auth) > 2:
        raise exceptions.PermissionDenied(_('Invalid Authorization header.'))

    jwt_value = auth[1]
    if jwt_value is None:
        return None

    try:
        payload = jwt_decode_handler(jwt_value)
    except jwt.ExpiredSignature:
        raise exceptions.PermissionDenied(_('Signature has expired.'))
    except jwt.DecodeError:
        raise exceptions.PermissionDenied(_('Error decoding signature.'))
    except jwt.InvalidTokenError:
        raise exceptions.PermissionDenied(_('Invalid token.'))

    return payload



# -*- coding: utf-8 -*-
from django.utils.lru_cache import lru_cache
from django.utils.translation import ugettext_lazy as _

from .base import ScopeProviderBase


class AdminScopeProvider(ScopeProviderBase):
    """
    Returns admin scopes
    """
    @lru_cache()
    def get_available_scopes(self):
        from rest_jwt_permission.scopes import SimpleScope
        return [
            SimpleScope("superuser", _("Super user access"))
        ]



# -*- coding: utf-8 -*-
import jwt
from django.conf.urls import include, url
from django.contrib.auth import get_user_model
from django.test.utils import override_settings
from django.utils.text import force_text
from rest_framework import response, routers, serializers, status, views, viewsets
from rest_framework.decorators import api_view, detail_route, list_route, permission_classes
from rest_framework.test import APIClient

from rest_jwt_permission.permissions import JWTAPIPermission
from rest_jwt_permission.providers import get_permission_providers


class SimpleSerializer(serializers.Serializer):
    pk = serializers.ReadOnlyField()


class BasicView(views.APIView):
    permission_classes = [JWTAPIPermission]

    def get(self, request):
        return response.Response({"method": "GET"})

    def post(self, request):
        return response.Response({"method": "POST", "data": request.data})


@api_view(http_method_names=["post", "get"])
@permission_classes([JWTAPIPermission])
def function_endpoint(request):
    return response.Response({"method": request.method})


class SimpleViewSetPermission(viewsets.ViewSet):
    queryset = get_user_model().objects.all()
    permission_classes = [JWTAPIPermission]

    @list_route(methods=["patch", "put"], permission_classes=[JWTAPIPermission])
    def patch_put(self, request):
        return response.Response({"method": request.method})


class ModelViewSetPermission(viewsets.ModelViewSet):
    queryset = get_user_model().objects.all()
    serializer_class = SimpleSerializer
    permission_classes = [JWTAPIPermission]

    def create(self, request):
        return response.Response({"method": request.method})

    def destroy(self, request, pk=None):
        return response.Response({"method": request.method})

    @list_route(methods=["delete", "get"], permission_classes=[JWTAPIPermission])
    def some_method(self, request):
        return response.Response({"method": request.method})

    @detail_route(methods=["get", "post", "patch", "put"], permission_classes=[JWTAPIPermission])
    def some_detail_metod(self, request, pk=None):
        return response.Response({"method": request.method})


router = routers.DefaultRouter()
router.register(r"simple_viewset", SimpleViewSetPermission)
router.register(r"model_viewset", ModelViewSetPermission)

urlpatterns = [
    url(r"^basic/$", BasicView.as_view()),
    url(r"^function/$", function_endpoint),
    url(r"^other/", include(router.urls)),
]

def setup_function(fn):
    get_permission_providers.cache_clear()


def test_jwt_api_endpoint_permission(admin_user):
    with override_settings(
        ROOT_URLCONF="rest_jwt_permission_tests.test_permissions",
        REST_JWT_PERMISSION={
            "SCOPE_PROVIDERS": [
                "rest_jwt_permission.providers.APIEndpointScopeProvider",
                "rest_jwt_permission.providers.AdminScopeProvider"
            ]
        }
    ):
        payload = {
            "scopes": [
                "function_endpoint:get",
                "basicview:get",
                "simpleviewsetpermission:patch_put:put",
                "modelviewsetpermission:retrieve:get",
                "modelviewsetpermission:create:post",
                "modelviewsetpermission:some_method:delete",
                "modelviewsetpermission:destroy:delete",
                "modelviewsetpermission:some_detail_metod:patch",
            ]
        }
        client = APIClient()

        assert client.get("/function/").status_code == status.HTTP_403_FORBIDDEN

        key = "super-secret"  # defined at conftest.py
        authorization = "JWT " + force_text(jwt.encode(payload, key, algorithm="HS256"))
        client.credentials(HTTP_AUTHORIZATION=authorization)

        assert client.get("/function/").status_code == status.HTTP_200_OK
        assert client.post("/function/").status_code == status.HTTP_403_FORBIDDEN

        assert client.get("/basic/").status_code == status.HTTP_200_OK
        assert client.post("/basic/").status_code == status.HTTP_403_FORBIDDEN

        assert client.put("/other/simple_viewset/patch_put/").status_code == status.HTTP_200_OK
        assert client.patch("/other/simple_viewset/patch_put/").status_code == status.HTTP_403_FORBIDDEN

        assert client.get("/other/model_viewset/{}/".format(admin_user.id)).status_code == status.HTTP_200_OK
        assert client.post("/other/model_viewset/", data={"data": 1}).status_code == status.HTTP_200_OK
        assert client.delete("/other/model_viewset/some_method/", data={"data": 2}).status_code == status.HTTP_200_OK
        assert client.delete("/other/model_viewset/{}/".format(admin_user.id), data={"data": 3}).status_code == status.HTTP_200_OK
        assert client.patch("/other/model_viewset/{}/some_detail_metod/".format(admin_user.id), data={"data": 4}).status_code == status.HTTP_200_OK



# -*- coding: utf-8 -*-
from django.utils.text import slugify

from rest_jwt_permission.utils import get_role_for, get_view_id, get_view_role
from rest_jwt_permission_tests.factories import BasicView


def test_get_role_for():
    assert get_role_for("post", "save") == "save:post"
    assert get_role_for("POST", "Save") == "Save:post"
    assert get_role_for("get") == "get"
    assert get_role_for("PATCH") == "patch"


class ViewWithClassMethod(BasicView):
    @classmethod
    def get_view_permission_id(cls):
        return "MyViewID_InClassMethod"


class ViewWithInstanceMethod(BasicView):
    def get_view_permission_id(self):
        return "MyViewID_InstanceMethod"


class ViewWithAnything(object):
    pass


def test_get_view_id():
    assert get_view_id(BasicView) == slugify(BasicView.__name__)
    assert get_view_id(BasicView()) == slugify(BasicView.__name__)

    assert get_view_id(ViewWithClassMethod) == slugify("MyViewID_InClassMethod")
    assert get_view_id(ViewWithClassMethod()) == slugify("MyViewID_InClassMethod")
    assert get_view_id(ViewWithInstanceMethod) == slugify("MyViewID_InstanceMethod")
    assert get_view_id(ViewWithInstanceMethod()) == slugify("MyViewID_InstanceMethod")

    assert get_view_id(ViewWithAnything) == slugify(ViewWithAnything.__name__)
    assert get_view_id(ViewWithAnything()) == slugify(ViewWithAnything.__name__)


def test_get_view_role():
    assert get_view_role(BasicView, "method1") == "{}:{}".format(slugify(BasicView.__name__), "method1")
    assert get_view_role(BasicView(), "method2") == "{}:{}".format(slugify(BasicView.__name__), "method2")

    assert get_view_role(ViewWithClassMethod, "method3") == "{}:{}".format(slugify("MyViewID_InClassMethod"), "method3")
    assert get_view_role(ViewWithClassMethod(), "method4") == "{}:{}".format(slugify("MyViewID_InClassMethod"), "method4")
    assert get_view_role(ViewWithInstanceMethod, "method5") == "{}:{}".format(slugify("MyViewID_InstanceMethod"), "method5")
    assert get_view_role(ViewWithInstanceMethod(), "method6") == "{}:{}".format(slugify("MyViewID_InstanceMethod"), "method6")

    assert get_view_role(ViewWithAnything, "method9") == "{}:{}".format(slugify(ViewWithAnything.__name__), "method9")
    assert get_view_role(ViewWithAnything(), "method10") == "{}:{}".format(slugify(ViewWithAnything.__name__), "method10")



# -*- coding: utf-8 -*-


class ScopeProviderBase(object):

    def get_available_scopes(self):
        """
        Returns a list of available Scope

        scope: is a string that uniquely identifies the permission level
        payload: additional content that might be serialized with format_payload()

        :rtype: list[rest_jwt_permission.Scope]
        """
        raise NotImplementedError()
